# https://www.cnblogs.com/superbaby11/p/15560994.html
from threading import Thread

def run_cmd(cmd):
    """Run shell command and return command output.
    
    Examples:
        >>> run_cmd("cd ..")
        ''
    
    Args:
        cmd (string): A shell command.
    
    Returns:
        string: Command output.
    """
    import subprocess
    return subprocess.getoutput(cmd)

def run_deno_cmd(cmd):
    from pathlib import Path
    return run_cmd(str(Path.home()) + "/.deno/bin/" + cmd)

def print_run_cmd(cmd):
    import subprocess
    # http://blog.kagesenshi.org/2008/02/teeing-python-subprocesspopen-output.html
    p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE,
                         stderr=subprocess.STDOUT, encoding='utf-8', errors='replace')
    stdout_buffer = []
    while True:
        line = p.stdout.readline()
        stdout_buffer.append(line)
        print(line, end='')
        if line == '' and p.poll() != None:
            break
    return ''.join(stdout_buffer)

def create_text_file(file_name, file_content):
    with open(file_name,"w") as file:
        file.write(file_content)

# https://www.jb51.net/article/232129.htm
class MyThread(Thread):
    def __init__(self, group=None, target=None, name=None, args=(), kwargs=None, *, daemon=None):
        super().__init__()
        self.fun = target
        self.args = args
        self.kwargs = kwargs
    def run(self):
        self.fun(*self.args, **self.kwargs)

def thread_run_cmd(cmd):
    t = MyThread(target=run_cmd, args=(cmd,), kwargs={})
    t.start()
    # t.join()
    
def thread_run_deno_cmd(cmd):
    t = MyThread(target=run_deno_cmd, args=(cmd,), kwargs={})
    t.start()
    # t.join()

def get_page_content(url):
    # https://www.cnblogs.com/herbert/p/10789343.html
    # https://www.selenium.dev/documentation/webdriver/drivers/remote_webdriver/#tabs-1-1
    from selenium import webdriver
    import time
    import os
    # https://blog.csdn.net/ad72182009/article/details/116117744
    if 'http_proxy' in os.environ:
        del os.environ['http_proxy']
    chrome_options = webdriver.ChromeOptions()
    driver = webdriver.Remote(
        command_executor='http://127.0.0.1:4444/wd/hub',
        options=chrome_options)
    driver.get(url=url)
    # Wait some time for page loading
    time.sleep(4)
    found_page_source = driver.page_source
    driver.close()
    return found_page_source

def display_code(content):
    from IPython.display import Code
    return Code(data=content)

def display_iframe(url):
    from IPython.display import IFrame
    return IFrame(src = url, width = "100%", height=300)

def get_file_md5sum(file):
    import hashlib
    import os
    if not os.path.exists(file):
        return "md5sum: {}: No such file or directory".format(file)
    elif os.path.isdir(file):
        return "md5sum: {}: Is a directory".format(file)
    elif os.path.isfile(file):
        with open(file, 'rb') as fd:
            data = fd.read()
        #return "{}  {}".format(hashlib.md5(data).hexdigest(), file)
        return hashlib.md5(data).hexdigest()
    else:
        return "md5sum: {}: Unexpected error".format(file)

def get_str_md5sum(content):
    import hashlib
    return hashlib.md5(content.encode('utf8')).hexdigest()

def append_text_file(file_name, file_content):
    with open(file_name,"a") as file:
        file.write(file_content)

def replace_line_in_file(file_path, old_line_number, new_line_content):
    import fileinput
    line_number = 0
    with fileinput.FileInput(file_path, inplace=True) as f:
        for line in f:
            line_number = line_number + 1;
            if line_number == old_line_number:
                print(new_line_content, end='')
            else:
                print(line, end='')

def insert_line_at_top_of_file(file_path, new_line_content, new_line_content_end='\n'):
    import fileinput
    flag_inserted = False
    with fileinput.FileInput(file_path, inplace=True) as f:
        for line in f:
            if flag_inserted == False:
                print(new_line_content, end=new_line_content_end)
                flag_inserted = True
            print(line, end='')

def add_line_in_file(file_path, old_line_number, new_line_content, new_line_content_end='\n'):
    import fileinput
    line_number = 0
    with fileinput.FileInput(file_path, inplace=True) as f:
        for line in f:
            line_number = line_number + 1;
            if line_number == old_line_number:
                print(new_line_content, end=new_line_content_end)
            print(line, end='')

def del_line_in_file(file_path, old_line_number):
    import fileinput
    line_number = 0
    with fileinput.FileInput(file_path, inplace=True) as f:
        for line in f:
            line_number = line_number + 1;
            if line_number == old_line_number:
                pass
            else:
                print(line, end='')

def read_text_file(file_path):
    from pathlib import Path
    return Path(file_path).read_text()

def display_text(content):
    from IPython.display import Code
    return Code(data=content, language="text")

def remove_indent_space_in_file(file_path, block_line_begin, block_line_end, indent_space_number):
    import fileinput
    line_number = 0
    with fileinput.FileInput(file_path, inplace=True) as f:
        for line in f:
            line_number = line_number + 1
            if line_number >= block_line_begin and line_number <= block_line_end:
                if len(line) < indent_space_number:
                    print(line, end='')
                else:
                    new_line_content = line[indent_space_number:]
                    print(new_line_content, end='')
            else:
                print(line, end='')

def rstrip_space_in_file(file_path, line_number):
    import fileinput
    cur_line_number = 0
    with fileinput.FileInput(file_path, inplace=True) as f:
        for line in f:
            cur_line_number = cur_line_number + 1
            if line_number == cur_line_number:
                print(line.rstrip() + "\n", end='')
            else:
                print(line, end='')

def display_image(img_file):
    from IPython.display import Image
    return Image(img_file)

def del_block_in_file(file_path, block_line_begin, block_line_end):
    import fileinput
    line_number = 0
    with fileinput.FileInput(file_path, inplace=True) as f:
        for line in f:
            line_number = line_number + 1;
            if (line_number >= block_line_begin and line_number <= block_line_end):
                pass
            else:
                print(line, end='')



import os
from py_book_util import util
import json

class ImageRecognizer(object):
    def __init__(self, img_name, crop_img_top, crop_img_bottom, crop_img_left, crop_img_right, cur_img_file, dst_img_dir, flag_force = False, ratio="2.0", flag_autocrop = True):
        self.img_name = img_name
        self.top = crop_img_top
        self.bottom = crop_img_bottom
        self.left = crop_img_left
        self.right = crop_img_right
        self.cur_img_file = cur_img_file
        self.dst_img_dir = dst_img_dir
        self.ratio = ratio
        self.flag_force = flag_force
        self.flag_autocrop = flag_autocrop
        self.prepare_img()
        
    def get_dst_img_file(self):
        return self.dst_img_dir + os.path.sep + self.img_name # "fr_sth3_interface_iosf_primary.png"
    
    def get_dst_img_file_json(self):
        return "%s.json" % self.get_dst_img_file()
    
    def prepare_img(self):
        dst_img_file = self.get_dst_img_file()
        if self.flag_force:
            if os.path.isfile(dst_img_file):
                os.remove(dst_img_file)
        if os.path.isfile(dst_img_file) == False:
            # cargo install menyoki
            util.print_run_cmd("menyoki edit %s --grayscale --crop %d:%d:%d:%d save %s" % (self.cur_img_file, self.top, self.right, self.bottom, self.left, dst_img_file))
            # util.print_run_cmd("menyoki edit %s --ratio 4.0 --filter gaussian save %s" % (dst_img_file, dst_img_file))
            if self.ratio != 1:
                util.print_run_cmd("menyoki edit %s --ratio %s --filter gaussian save %s" % (dst_img_file, self.ratio, dst_img_file))
            # cargo install auto-image-cropper
            if self.flag_autocrop:
                util.print_run_cmd("autocrop -i %s -o %s" % (dst_img_file, dst_img_file))
        return dst_img_file

    def recognize_text(self, app_name="img2txt"):
        dst_img_file_json = self.get_dst_img_file_json()
        # print(dst_img_file_json)
        if self.flag_force:
            if os.path.isfile(dst_img_file_json):
                os.remove(dst_img_file_json)
        if os.path.isfile(dst_img_file_json) == False:
            # util.run_deno_cmd("js_book_util img2txt %s" % dst_img_file)
            # util.run_deno_cmd("js_book_util easyocr2txt %s" % dst_img_file)
            util.run_deno_cmd("js_book_util %s %s" % (app_name, self.get_dst_img_file()))
            data = json.load(open(dst_img_file_json, 'r', encoding="utf-8"))
            print(data['recognize_text'])
            data['crop_img_top'] = self.top
            data['crop_img_bottom'] = self.bottom
            data['crop_img_left'] = self.left
            data['crop_img_right'] = self.right
            with(open(dst_img_file_json, 'w') as fp):
                json.dump(data, fp)
        else:
            data = json.load(open(dst_img_file_json, 'r', encoding="utf-8"))
        return data



from py_book_util import util
from py_book_util.image_recognizer import ImageRecognizer
import os
import json
from myst_nb import glue
class PageRecognizer(object):
    def __init__(self, cur_pages_dir, cur_page_idx, cur_page_width=2561, cur_page_height=1206):
        self.page_width = cur_page_width
        self.page_height = cur_page_height
        self.pages_dir = cur_pages_dir
        self.page_idx = cur_page_idx
        self.page_elements = {}
        dst_img_dir = self.dst_img_dir()
        if os.path.isdir(dst_img_dir) == False:
            os.mkdir(dst_img_dir)
    def cur_img_file(self):
        return "%s/%s.png" % (self.pages_dir, self.page_idx)
    def dst_img_dir(self):
        return "%s/%s" % (self.pages_dir, self.page_idx)
    def recognize_rect(self, cur_key_word, crop_img_top, crop_img_bottom, crop_img_left, crop_img_right, post_replace={}, flag_force = False, ratio="2.0", flag_autocrop = True, app_name="img2txt"):
        cur_rec = ImageRecognizer("%s.png"%cur_key_word, crop_img_top, crop_img_bottom, crop_img_left, crop_img_right, self.cur_img_file(), self.dst_img_dir(), flag_force, ratio, flag_autocrop)
        cur_rec_data = cur_rec.recognize_text(app_name)
        for single_replace_key in post_replace:
            cur_rec_data['recognize_text'] = cur_rec_data['recognize_text'].replace(single_replace_key, post_replace[single_replace_key]) # ("\u2014", "-")        
        glue(cur_key_word, cur_rec_data['recognize_text'])
        self.page_elements[cur_key_word] = cur_rec_data        
        return cur_rec

    def generate_html_js(self, element_height={}):
        self.generate_js(element_height)
        self.generate_html()
        
    def generate_js(self, element_height={}):
        file_name_page_js = "%s.js"%self.cur_img_file()
        if os.path.isfile(file_name_page_js):
            os.remove(file_name_page_js)
        cur_page_elements_js = ""
        # cur_page_ratio = cur_page_width / 1280
        cur_page_element_top = 0
        cur_page_element_height = 0
        cur_page_element_idx = 0
        for single_element in self.page_elements:
            cur_page_element_idx = cur_page_element_idx + 1
            cur_page_element_top = cur_page_element_top + cur_page_element_height + 10
            cur_page_element_height = 60
            if single_element in element_height :
                cur_page_element_height = element_height[single_element]
            single_data = self.page_elements[single_element]
            print(single_element)
            print(single_data)
            cur_page_elements_js = cur_page_elements_js + """
image_note.add(new DoneElement(%d / 2, %d / 2, %d / 2, %d / 2, "%s<br/>%s"));
""" % (single_data["crop_img_left"], single_data["crop_img_top"], 
       cur_page_width - single_data["crop_img_right"] - single_data["crop_img_left"], 
       cur_page_height - single_data["crop_img_bottom"] - single_data["crop_img_top"], 
       single_element, single_data["recognize_text"])
            cur_page_elements_js = cur_page_elements_js + """
image_note.add(new CommentElement(1300, 30+%d, 250, 44+%d, "Note_%s_%03d:<br/> - %s<br/> - %s"));
""" % (cur_page_element_top, cur_page_element_height - 44, self.page_idx, cur_page_element_idx, single_element, single_data["recognize_text"])
        file_content_page_js = """
var currentScript = $('script').last();

$(document).ready(function () {
	var image_file = "%s.png";
  var image_index = "000000";
  var image_note = new ImageNote(image_index, image_file, 0, 0);
  // <div style="width:730px; height:420px; position:absolute; left: 215px; top: 10px; color:#FFF; background:#0E0;filter:alpha(opacity=30); opacity:0.3; -moz-opacity=0.3; z-index:1;" id="Done_000000_001">&nbsp;</div>
  // <div style="width:250px; height:44px; position:absolute; left: 980px; top: 30px; color:#FFF; background:#0C3;filter:alpha(opacity=50); opacity:0.6; -moz-opacity=0.6; z-index:1;" id="Note_000000_001">Note_000000_001:<br/>Author is Carrieri, Enrico D</div>
  
  %s  

  image_note.create();
});
""" % (self.page_idx, cur_page_elements_js)
        util.create_text_file(file_name_page_js, file_content_page_js)
        # util.display_code(file_name_page_js)
        print("http://localhost:4507/xxx_dev_blog/%s/%s.png.html" % (cur_pages_dir, self.page_idx))

    def generate_html(self):
        file_name_page_html = "%s.html"%self.cur_img_file()
        if os.path.isfile(file_name_page_html):
            os.remove(file_name_page_html)
        file_content_page_html = """<html>
  <head>
    <title>%s</title>
    <script src="../../static/note_common.js"></script>
    <script src="../../static/jquery-3.6.1.min.js"></script>
  </head>
  <body>
    <div style="width:1280px;">
      <div>
        <img width="1280" class="image_notes" src="%s.png"></img>
        <script src="%s.png.js"></script>
      </div>
    </div>
  </body>
</html>
""" % (self.page_idx, self.page_idx, self.page_idx)
        util.create_text_file(file_name_page_html, file_content_page_html)
        # util.display_code(file_name_page_html)
        # print("http://localhost:4507/xxx_dev_blog/%s/%s.png.html" % (cur_pages_dir, self.page_idx))
